@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -300,8 +301,8 @@ protected:
    typedef AVL::Ptr<Node> Ptr;
 
 private:
-   /// this is private since direct assignment is forbidden
-   void operator= (const tree&);
+   /// direct assignment is forbidden
+   void operator= (const tree&) = delete;
 
    /** Recursive procedure making an exact copy of the tree.
        @param n        root node of the subtree to be cloned
@@ -450,7 +451,7 @@ private:
       }
       link_index Down;
       cur=root_node();
-      typename Diligent<const Key&>::type K=diligent(k);
+      const auto& K=diligent(k);
       while (1) {
          Down=link_index(comparator(K, this->key(*cur)));
          if (Down==P ||
@@ -481,10 +482,10 @@ private:
    }
 
    template <typename Key>
-   void _assign_node(Node&, const Key&, False) {}
+   void assign_node(Node&, const Key&, std::false_type) {}
 
    template <typename Key, typename Data>
-   void _assign_node(Node& n, const pair<Key, Data>& p, True)
+   void assign_node(Node& n, const pair<Key, Data>& p, std::true_type)
    {
       Traits::data(n)=p.second;
    }
@@ -500,7 +501,7 @@ private:
       if (!n_elem) return insert_first(this->create_node(k));
       pair<Ptr,link_index> p=find_descend(k, this->key_comparator);
       if (p.second==P) {
-         _assign_node(*p.first, k, bool2type<isomorphic_to_first<key_type, Key>::value>());
+         assign_node(*p.first, k, bool_constant<isomorphic_to_first<key_type, Key>::value>());
          return p.first;
       }
       ++n_elem;
@@ -515,12 +516,12 @@ private:
       if (!n_elem) return insert_first(this->create_node(k,d));
       pair<Ptr,link_index> p=find_descend(k, this->key_comparator);
       if (p.second==P) {
-         op(Traits::data(*p.first),d);
+         op(Traits::data(*p.first), d);
          return p.first;
       }
       ++n_elem;
       Node *n=this->create_node(k,d);
-      insert_rebalance(n,p.first,p.second);
+      insert_rebalance(n, p.first, p.second);
       return n;
    }
 
@@ -554,15 +555,13 @@ public:
    {
       if (n_elem) {
          if (is_instance_of<Key, maximal>::value) {
-            if (identical<RelOp, polymake::operations::le>::value ||
-                identical<RelOp, polymake::operations::lt>::value)
+            if (is_among<RelOp, polymake::operations::le, polymake::operations::lt>::value)
                return last();
             else
                return end_node();
          }
          if (is_instance_of<Key, minimal>::value) {
-            if (identical<RelOp, polymake::operations::ge>::value ||
-                identical<RelOp, polymake::operations::gt>::value)
+            if (is_among<RelOp, polymake::operations::ge, polymake::operations::gt>::value)
                return first();
             else
                return end_node();
@@ -572,38 +571,28 @@ public:
          switch (p.second) {
          case P:
             if (Traits::allow_multiple) {
-               if (identical<RelOp, last_of_equal>::value ||
-                   identical<RelOp, polymake::operations::ge>::value ||
-                   identical<RelOp, polymake::operations::gt>::value) {
+               if (is_among<RelOp, last_of_equal, polymake::operations::ge, polymake::operations::gt>::value)
                   find_last_equal(p.first, R);
-               }
-               if (identical<RelOp, first_of_equal>::value ||
-                   identical<RelOp, polymake::operations::le>::value ||
-                   identical<RelOp, polymake::operations::lt>::value) {
+               if (is_among<RelOp, first_of_equal, polymake::operations::le, polymake::operations::lt>::value)
                   find_last_equal(p.first, L);
-               }
             }
-            if (identical<RelOp, polymake::operations::gt>::value)
+            if (std::is_same<RelOp, polymake::operations::gt>::value)
                p.first.traverse(*this,R);
-            if (identical<RelOp, polymake::operations::lt>::value)
+            if (std::is_same<RelOp, polymake::operations::lt>::value)
                p.first.traverse(*this,L);
             return p.first;
 
          case R:
-            if (identical<RelOp, polymake::operations::le>::value ||
-                identical<RelOp, polymake::operations::lt>::value)
+            if (is_among<RelOp, polymake::operations::le, polymake::operations::lt>::value)
                return p.first;
-            if (identical<RelOp, polymake::operations::ge>::value ||
-                identical<RelOp, polymake::operations::gt>::value)
+            if (is_among<RelOp, polymake::operations::ge, polymake::operations::gt>::value)
                return p.first.traverse(*this,R);
             break;
 
          case L:
-            if (identical<RelOp, polymake::operations::ge>::value ||
-                identical<RelOp, polymake::operations::gt>::value)
+            if (is_among<RelOp, polymake::operations::ge, polymake::operations::gt>::value)
                return p.first;
-            if (identical<RelOp, polymake::operations::le>::value ||
-                identical<RelOp, polymake::operations::lt>::value)
+            if (is_among<RelOp, polymake::operations::le, polymake::operations::lt>::value)
                return p.first.traverse(*this,L);
             break;
          }
@@ -615,8 +604,7 @@ public:
    Ptr find_nearest_node(const Key& k, const RelOp&, const Comparator& comparator) const
    {
       Ptr found=end_node();
-      const link_index Left= identical<RelOp, polymake::operations::ge>::value ||
-                             identical<RelOp, polymake::operations::gt>::value ? L : R,
+      const link_index Left= is_among<RelOp, polymake::operations::ge, polymake::operations::gt>::value ? L : R,
                        Right=link_index(-Left);
       link_index Down;
       if (n_elem) {
@@ -631,7 +619,7 @@ public:
                return found;
             treeify();
          }
-         typename Diligent<const Key&>::type K=diligent(k);
+         const auto& K=diligent(k);
          Ptr cur=root_node();
          do {
             const cmp_value diff=comparator(this->key(*cur), K);
@@ -678,80 +666,80 @@ public:
       return n;
    }
 
+   template <typename Arg>
+   struct insert_arg_helper {
+      static const bool is_reverse_iterator=is_derived_from<Arg, reverse_iterator>::value || is_derived_from<Arg, const_reverse_iterator>::value;
+      static const bool is_iterator=is_derived_from<Arg, iterator>::value || is_derived_from<Arg, const_iterator>::value || is_reverse_iterator;
+      static const bool is_pair=isomorphic_to_first<key_type, Arg>::value;
+      static const int d=is_iterator*2+is_pair;
+      using discr = int_constant<d>;
+      using return_type = std::conditional_t<is_reverse_iterator, reverse_iterator, iterator>;
+   };
+
    template <typename Key>
-   iterator _insert(const Key& k, int2type<0>)
+   iterator insert_impl(const Key& k, int_constant<0>)
    {
       return iterator(this->get_it_traits(), find_insert(k));
    }
 
    template <typename Key, typename Data>
-   iterator _insert(const Key& k, const Data& data, int2type<0>)
+   iterator insert_impl(const Key& k, const Data& data, int_constant<0>)
    {
       return iterator(this->get_it_traits(), find_insert(k, data, assign_op()));
    }
 
    template <typename Key, typename Data, typename Operation>
-   iterator _insert(const Key& k, const Data& data, const Operation& op, int2type<0>)
+   iterator insert_impl(const Key& k, const Data& data, const Operation& op, int_constant<0>)
    {
       return iterator(this->get_it_traits(), find_insert(k, data, op));
    }
 
    template <typename Key, typename Data>
-   iterator _insert(const pair<Key, Data>& p, int2type<1>)
+   iterator insert_impl(const pair<Key, Data>& p, int_constant<1>)
    {
       return iterator(this->get_it_traits(), find_insert(p.first, p.second, assign_op()));
    }
 
    template <typename Key, typename Data, typename Operation>
-   iterator _insert(const pair<Key, Data>& p, const Operation& op, int2type<1>)
+   iterator insert_impl(const pair<Key, Data>& p, const Operation& op, int_constant<1>)
    {
       return iterator(this->get_it_traits(), find_insert(p.first, p.second, op));
    }
 
-   template <typename Key>
-   iterator _insert(const iterator& pos, const Key& k, int2type<2>)
+   template <typename Iterator, typename Key>
+   auto insert_impl(const Iterator& pos, const Key& k, int_constant<2>)
    {
-      return iterator(this->get_it_traits(), insert_node_at(pos.cur, L, this->create_node(k)));
+      using result_type = std::conditional_t<insert_arg_helper<Iterator>::is_reverse_iterator, reverse_iterator, iterator>;
+      return result_type(this->get_it_traits(), insert_node_at(pos.cur, insert_arg_helper<Iterator>::is_reverse_iterator ? R : L, this->create_node(k)));
    }
 
-   template <typename Key, typename Data>
-   iterator _insert(const iterator& pos, const Key& k, const Data& data, int2type<2>)
+   template <typename Iterator, typename Key, typename Data>
+   auto insert_impl(const Iterator& pos, const Key& k, const Data& data, int_constant<2>)
    {
-      return iterator(this->get_it_traits(), insert_node_at(pos.cur, L, this->create_node(k, data)));
+      using result_type = std::conditional_t<insert_arg_helper<Iterator>::is_reverse_iterator, reverse_iterator, iterator>;
+      return result_type(this->get_it_traits(), insert_node_at(pos.cur, insert_arg_helper<Iterator>::is_reverse_iterator ? R : L, this->create_node(k, data)));
    }
 
    template <typename Key>
-   reverse_iterator _insert(const reverse_iterator& pos, const Key& k, int2type<2>)
-   {
-      return reverse_iterator(this->get_it_traits(), insert_node_at(pos.cur, R, this->create_node(k)));
-   }
-
-   template <typename Key, typename Data>
-   reverse_iterator _insert(const reverse_iterator& pos, const Key& k, const Data& data, int2type<2>)
-   {
-      return reverse_iterator(this->get_it_traits(), insert_node_at(pos.cur, R, this->create_node(k, data)));
-   }
-
-   template <typename Key>
-   iterator _insert_new(const Key& k, int2type<0>)
+   iterator insert_new_impl(const Key& k, int_constant<0>)
    {
       return iterator(this->get_it_traits(), insert_node(this->create_node(k)));
    }
 
    template <typename Key, typename Data>
-   iterator _insert_new(const Key& k, const Data& data, int2type<0>)
+   iterator insert_new_impl(const Key& k, const Data& data, int_constant<0>)
    {
       return iterator(this->get_it_traits(), insert_node(this->create_node(k, data)));
    }
 
    template <typename Key, typename Data>
-   iterator _insert_new(const pair<Key, Data>& p, int2type<1>)
+   iterator insert_new_impl(const pair<Key, Data>& p, int_constant<1>)
    {
       return iterator(this->get_it_traits(), insert_node(this->create_node(p.first, p.second)));
    }
 
    template <typename Key>
-   void _erase(const Key& k, int2type<0>)
+   void erase_impl(const Key& k, int_constant<0>)
    {
       if (empty()) return;
       pair<Ptr,link_index> p=find_descend(k, this->key_comparator);
@@ -760,17 +748,14 @@ public:
       this->destroy_node(p.first);
    }
 
-   void _erase(const iterator& pos, int2type<2>)
+   template <typename Iterator>
+   void erase_impl(const Iterator& pos, int_constant<2>)
    {
       this->destroy_node(remove_node(pos.cur));
    }
 
-   void _erase(const reverse_iterator& pos, int2type<2>)
-   {
-      this->destroy_node(remove_node(pos.cur));
-   }
-
-   void _erase(iterator pos, const iterator& end, int2type<2>)
+   template <typename Iterator>
+   void erase_impl(Iterator pos, const Iterator& end, int_constant<2>)
    {
       while (pos != end) {
          Node *n=pos.cur;  ++pos;
@@ -778,24 +763,6 @@ public:
       }
    }
 
-   void _erase(reverse_iterator pos, const reverse_iterator& end, int2type<2>)
-   {
-      while (pos != end) {
-         Node *n=pos.cur;  ++pos;
-         this->destroy_node(remove_node(n));
-      }
-   }
-
-   void swap_nodes_list_form(Node *l, Node *r)
-   {
-      Ptr pl=this->link(l,L), pr=this->link(r,L);
-      std::swap(this->link(pl,R), this->link(pr,R));
-      this->link(l,L)=pr;  this->link(r,L)=pl;
-      pl=this->link(l,R);  pr=this->link(r,R);
-      std::swap(this->link(pl,L), this->link(pr,L));
-      this->link(l,R)=pr;  this->link(r,R)=pl;
-   }
-
 public:
    template <typename Key>
    iterator find(const Key& k)
@@ -851,51 +818,54 @@ public:
       return ! find_node(k,this->key_comparator).end();
    }
 
-   template <typename Arg>
-   struct insert_arg_helper {
-      static const bool is_reverse_iterator=derived_from<Arg, reverse_iterator>::value;
-      static const bool is_iterator=derived_from<Arg, iterator>::value || is_reverse_iterator;
-      static const bool is_pair=isomorphic_to_first<key_type, Arg>::value;
-      static const int d=is_iterator*2+is_pair;
-      typedef int2type<d> discr;
-      typedef typename if_else<is_reverse_iterator, reverse_iterator, iterator>::type return_type;
-   };
-
    template <typename First>
    typename insert_arg_helper<First>::return_type
    insert(const First& first_arg)
    {
-      return _insert(first_arg, typename insert_arg_helper<First>::discr());
+      return insert_impl(first_arg, typename insert_arg_helper<First>::discr());
    }
 
    template <typename First, typename Second>
    typename insert_arg_helper<First>::return_type
    insert(const First& first_arg, const Second& second_arg)
    {
-      return _insert(first_arg, second_arg, typename insert_arg_helper<First>::discr());
+      return insert_impl(first_arg, second_arg, typename insert_arg_helper<First>::discr());
    }
 
    template <typename First, typename Second, typename Third>
    typename insert_arg_helper<First>::return_type
    insert(const First& first_arg, const Second& second_arg, const Third& third_arg)
    {
-      return _insert(first_arg, second_arg, third_arg, typename insert_arg_helper<First>::discr());
+      return insert_impl(first_arg, second_arg, third_arg, typename insert_arg_helper<First>::discr());
    }
 
    template <typename First>
-   typename enable_if<typename insert_arg_helper<First>::return_type, Traits::allow_multiple>::type
+   std::enable_if_t<!Traits::allow_multiple, typename insert_arg_helper<First>::return_type>
    insert_new(const First& first_arg)
    {
-      return _insert_new(first_arg, typename insert_arg_helper<First>::discr());
+      return insert_impl(first_arg, typename insert_arg_helper<First>::discr());
    }
 
    template <typename First, typename Second>
-   typename enable_if<typename insert_arg_helper<First>::return_type, Traits::allow_multiple>::type
+   std::enable_if_t<!Traits::allow_multiple, typename insert_arg_helper<First>::return_type>
    insert_new(const First& first_arg, const Second& second_arg)
    {
-      return _insert_new(first_arg, second_arg, typename insert_arg_helper<First>::discr());
+      return insert_impl(first_arg, second_arg, typename insert_arg_helper<First>::discr());
+   }
+
+   template <typename First>
+   std::enable_if_t<Traits::allow_multiple, typename insert_arg_helper<First>::return_type>
+   insert_new(const First& first_arg)
+   {
+      return insert_new_impl(first_arg, typename insert_arg_helper<First>::discr());
+   }
+
+   template <typename First, typename Second>
+   std::enable_if_t<Traits::allow_multiple, typename insert_arg_helper<First>::return_type>
+   insert_new(const First& first_arg, const Second& second_arg)
+   {
+      return insert_new_impl(first_arg, second_arg, typename insert_arg_helper<First>::discr());
    }
-   
 
    /// Insert a given node in the tree corresponding to the value of its key field.
    /// @return NULL if the key of the given node already occurs in the tree, //n// otherwise.
@@ -922,7 +892,7 @@ public:
                }
             }
          } else if (p.second == P) {
-            return NULL;
+            return nullptr;
          }
          ++n_elem;
          insert_rebalance(n,p.first,p.second);
@@ -937,13 +907,13 @@ public:
    template <typename Key_or_Iterator>
    void erase(const Key_or_Iterator& k_or_it)
    {
-      _erase(k_or_it, typename insert_arg_helper<Key_or_Iterator>::discr());
+      erase_impl(k_or_it, typename insert_arg_helper<Key_or_Iterator>::discr());
    }
 
    template <typename Iterator>
    void erase(const Iterator& pos, const Iterator& last)
    {
-      _erase(pos, last, typename insert_arg_helper<Iterator>::discr());
+      erase_impl(pos, last, typename insert_arg_helper<Iterator>::discr());
    }
 
    /** Remove the node with a given address.
@@ -1082,7 +1052,7 @@ public:
 
 private:
    template <bool force>
-   void destroy_nodes(bool2type<force>)
+   void destroy_nodes(bool_constant<force>)
    {
       Ptr n=last();
       do {
@@ -1098,7 +1068,7 @@ public:
    void clear()
    {
       if (n_elem) {
-         destroy_nodes(True());
+         destroy_nodes(std::true_type());
          init();
       }
    }
@@ -1106,12 +1076,18 @@ public:
    /// Creates an empty tree.
    tree() { init(); }
 
-   tree(typename Traits::arg_type traits_arg) : Traits(traits_arg) { init(); }
+   template <typename... Args, typename=std::enable_if_t<std::is_constructible<Traits, Args...>::value>>
+   explicit tree(Args&&... args)
+      : Traits(std::forward<Args>(args)...)
+   {
+      init();
+   }
 
    /** Create a copy of another tree.
        The current form (list or tree) is also inherited.
    */
-   tree(const tree& t) : Traits(t)
+   tree(const tree& t)
+      : Traits(t)
    {
       if (t.tree_form()) {
          n_elem=t.n_elem;
@@ -1127,42 +1103,35 @@ public:
 
 private:
    template <typename Iterator>
-   void _fill(Iterator& src, True)
+   void fill_impl(Iterator& src, std::true_type)
    {
       for (; !src.at_end(); ++src)
          push_back(src.index(), *src);
    }
 
    template <typename Iterator>
-   void _fill(Iterator& src, False)
+   void fill_impl(Iterator& src, std::false_type)
    {
       for (; !src.at_end(); ++src)
          push_back(*src);
    }
 
    template <typename Iterator>
-   void _fill(Iterator& src)
+   void fill_impl(Iterator&& src)
    {
-      _fill(src, bool2type< check_iterator_feature<Iterator,indexed>::value &&
-                            isomorphic_types<typename iterator_traits<Iterator>::value_type,
-                                             typename Node::mapped_type>::value >());
+      fill_impl(src, bool_constant< check_iterator_feature<Iterator, indexed>::value &&
+                                    isomorphic_types<typename iterator_traits<Iterator>::value_type, typename Node::mapped_type>::value >());
    }
 public:
    template <typename Iterator>
-   tree(Iterator src, Iterator end)
+   explicit tree(Iterator&& src,
+                 std::enable_if_t<assess_iterator<Iterator, check_iterator_feature, end_sensitive>::value, void**> =nullptr)
    {
       init();
-      for (; src != end; ++src)
-         push_back(*src);
+      fill_impl(ensure_private_mutable(std::forward<Iterator>(src)));
    }
 
-   template <typename Iterator>
-   explicit tree(Iterator src, typename enable_if_iterator<Iterator,end_sensitive>::type=0)
-   {
-      init(); _fill(src);
-   }
-
-   ~tree() { if (n_elem) destroy_nodes(False()); }
+   ~tree() { if (n_elem) destroy_nodes(std::false_type()); }
 
    void swap(tree& t)
    {
@@ -1171,11 +1140,11 @@ public:
       std::swap(n_elem, t.n_elem);
    }
 
-   template <bool _with_nodes>
-   friend void relocate_tree(tree* from, tree* to, bool2type<_with_nodes>)
+   template <bool with_nodes>
+   friend void relocate_tree(tree* from, tree* to, bool_constant<with_nodes>)
    {
       relocate(static_cast<Traits*>(from), static_cast<Traits*>(to));
-      if (_with_nodes && from->n_elem) {
+      if (with_nodes && from->n_elem) {
          to->n_elem=from->n_elem;
          to->link(to->first(),L)=to->link(to->last(),R)=to->end_node();
          if (to->tree_form())
@@ -1187,11 +1156,16 @@ public:
 
    friend void relocate(tree* from, tree* to)
    {
-      relocate_tree(from,to,True());
+      relocate_tree(from, to, std::true_type());
    }
 
-   template <typename Iterator>
-   void assign(Iterator src) { clear(); _fill(src); }
+   template <typename Iterator,
+             typename=std::enable_if_t<assess_iterator<Iterator, check_iterator_feature, end_sensitive>::value>>
+   void assign(Iterator&& src)
+   {
+      clear();
+      fill_impl(ensure_private_mutable(std::forward<Iterator>(src)));
+   }
 
 #if POLYMAKE_DEBUG
 private:
@@ -1511,13 +1485,13 @@ void tree<Traits>::remove_rebalance(Node *n)
 }
 
 template <typename Traits>
-void tree<Traits>::update_node(Node *n)
+void tree<Traits>::update_node(Node* n)
 {
    if (n_elem<=1) return;
 
    if (tree_form()) {
       Ptr l(n), r(n);
-      l.traverse(*this,L); r.traverse(*this,R);
+      l.traverse(*this,L); r.traverse(*this, R);
       if ((!l.end() && this->key_comparator(this->key(*l), this->key(*n))==cmp_gt) ||
           (!r.end() && this->key_comparator(this->key(*r), this->key(*n))==cmp_lt)) {
          --n_elem;
@@ -1526,25 +1500,30 @@ void tree<Traits>::update_node(Node *n)
       }
 
    } else {
-      Ptr next(n);
-      do
-         next=this->link(next,L);
-      while (!next.end() && this->key_comparator(this->key(*next), this->key(*n)) == cmp_gt);
+      Ptr old_prev=this->link(n, L),
+          old_next=this->link(n, R);
+      Ptr prev=old_prev,
+          next=old_next;
+      while (!prev.end() && this->key_comparator(this->key(*prev), this->key(*n)) == cmp_gt)
+         prev=this->link(prev, L);
 
-      next=this->link(next,R);
-      if (next != n) {
-         swap_nodes_list_form(next,n);
-         return;
+      if (prev != old_prev) {
+         next=this->link(prev, R);
+      } else {
+         while (!next.end() && this->key_comparator(this->key(*n), this->key(*next)) == cmp_gt)
+            next=this->link(next, R);
+         if (next != old_next)
+            prev=this->link(next, L);
+         else
+            return;
       }
 
-      do
-         next=this->link(next,R);
-      while (!next.end() && this->key_comparator(this->key(*n), this->key(*next)) == cmp_gt);
-
-      next=this->link(next,L);
-      if (next != n) {
-         swap_nodes_list_form(n,next);
-      }
+      this->link(old_prev, R)=old_next;
+      this->link(old_next, L)=old_prev;
+      this->link(prev, R)=n;
+      this->link(next, L)=n;
+      this->link(n, L)=prev;
+      this->link(n, R)=next;
    }
 }
 
@@ -1585,7 +1564,7 @@ struct node {
 /** Key-data pair access filter for @c node.
     Key field is referred to as @c first, data field as @c second.
 */
-template <typename NodeRef, bool _no_data=identical<typename deref<NodeRef>::type::mapped_type, nothing>::value>
+template <typename NodeRef, bool no_data=std::is_same<typename deref<NodeRef>::type::mapped_type, nothing>::value>
 struct node_accessor_impl {
    typedef NodeRef argument_type;
    typedef typename inherit_ref<typename deref<NodeRef>::type::value_type, NodeRef>::type result_type;
@@ -1602,7 +1581,7 @@ struct node_accessor_impl<NodeRef, true> {
 template <typename NodeRef>
 struct node_accessor : node_accessor_impl<NodeRef> {};
 
-template <typename K, typename D, typename Comparator>
+template <typename K, typename D>
 struct it_traits {
    typedef node<K,D> Node;
 
@@ -1611,15 +1590,16 @@ struct it_traits {
    const it_traits& get_it_traits() const { return *this; }
 };
 
-template <typename K, typename D, typename Comparator>
-class traits : public it_traits<K,D,Comparator> {
+template <typename K, typename D, typename... Params>
+class traits : public it_traits<K, D> {
 public:
-   typedef it_traits<K,D,Comparator> traits_for_iterator;
-   typedef typename traits_for_iterator::Node Node;
-   typedef K key_type;
-   typedef Comparator key_comparator_type;
-   static const bool allow_multiple=false;
-   typedef std::allocator<Node> node_allocator_type;
+   using traits_for_iterator = it_traits<K, D>;
+   using Node = typename traits_for_iterator::Node;
+   using key_type = K;
+   using params = typename mlist_wrap<Params...>::type;
+   using key_comparator_type = typename mtagged_list_extract<params, ComparatorTag, operations::cmp>::type;
+   static const bool allow_multiple = tagged_list_extract_integral<params, MultiTag>(false);
+   using node_allocator_type = std::allocator<Node>;
 
 protected:
    mutable Ptr<Node> root_links[3];
@@ -1633,21 +1613,21 @@ protected:
    static D& data(Node& n) { return n.key_and_data.second; }
 
    template <typename Key, typename Data>
-   Node* _create_from_pair(const pair<Key,Data>& arg, False)
+   Node* create_from_pair(const pair<Key, Data>& arg, std::false_type)
    {
       return new(node_allocator.allocate(1)) Node(arg.first, arg.second);
    }
 
    template <typename Key1, typename Key2>
-   Node* _create_from_pair(const pair<Key1,Key2>& arg, True)
+   Node* create_from_pair(const pair<Key1, Key2>& arg, std::true_type)
    {
       return new(node_allocator.allocate(1)) Node(arg, nothing());
    }
 
    template <typename Key1, typename Key2>
-   Node* create_node(const pair<Key1,Key2>& arg)
+   Node* create_node(const pair<Key1, Key2>& arg)
    {
-      return _create_from_pair(arg, bool2type<isomorphic_types<pair<Key1,Key2>, key_type>::value>());
+      return create_from_pair(arg, bool_constant<isomorphic_types<pair<Key1, Key2>, key_type>::value>());
    }
 
    template <typename Key>
@@ -1689,10 +1669,20 @@ public:
    const key_comparator_type& get_comparator() const { return key_comparator; }
 };
 
+template <typename K, typename D, typename Comparator>
+struct single_key_traits {
+   using type = traits<K, D, ComparatorTag<Comparator>>;
+};
+
+template <typename K, typename D>
+struct single_key_traits<K, D, operations::cmp> {
+   using type = traits<K, D>;
+};
+
 } // end namespace AVL
 
 template <typename Traits, AVL::link_index Dir>
-struct check_iterator_feature<AVL::tree_iterator<Traits,Dir>, end_sensitive> : True {};
+struct check_iterator_feature<AVL::tree_iterator<Traits,Dir>, end_sensitive> : std::true_type {};
 
 template <typename Traits, AVL::link_index Dir>
 struct iterator_reversed< AVL::tree_iterator<Traits,Dir> > {
