@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -862,8 +863,6 @@ pow(const GenericMatrix<TMatrix>& base, long exp)
    return pow_impl<typename TMatrix::persistent_nonsymmetric_type>(base, one, exp);
 }
 
-
-
 } // end namespace pm
 
 namespace polymake {
