@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,7 +26,7 @@ template <typename Matrix>
 bool detect_multiple(const GenericMatrix<Matrix> &mat)
 {
    hash_set<typename Matrix::const_row_type> all;
-   for (typename Entire<Rows<Matrix> >::const_iterator v=entire(rows(mat)); !v.at_end(); ++v)
+   for (auto v=entire(rows(mat)); !v.at_end(); ++v)
       if (!all.insert(*v).second) return true;
    return false;
 }
