@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -76,7 +77,7 @@ perl::Object simplex(int d, const Scalar& s, perl::OptionSet options)
    if (s==0)
       throw std::runtime_error("scale must be non-zero");
 
-   perl::Object p(perl::ObjectType::construct<Scalar>("Polytope"));
+   perl::Object p("Polytope", mlist<Scalar>());
    p.set_description() << "standard simplex of dimension " << d << endl;
 
    SparseMatrix<Scalar> V( ones_vector<Scalar>(d+1) | (zero_vector<Scalar>(d) / (s*unit_matrix<Scalar>(d))));
@@ -102,7 +103,7 @@ perl::Object regular_simplex(const int d, perl::OptionSet options)
    p.set_description() << "regular simplex of dimension " << d << endl;
 
    QE c(Rational(1,d),Rational(-1,d),d+1);
-   SparseMatrix<QE> V( ones_vector<QE>(d+1) | (unit_matrix<QE>(d) / same_element_vector<QE>(c,d)));
+   SparseMatrix<QE> V( ones_vector<QE>(d+1) | (unit_matrix<QE>(d) / same_element_vector(c,d)));
 
    p.take("VERTICES") << V;
    p.take("CONE_AMBIENT_DIM") << d+1;
@@ -118,7 +119,7 @@ perl::Object fano_simplex(int d, perl::OptionSet options)
    if (d <= 0)
       throw std::runtime_error("fano_simplex : dimension must be postive");
 
-   perl::Object p(perl::ObjectType::construct<Rational>("Polytope"));
+   perl::Object p("Polytope<Rational>");
    p.set_description() << "Fano simplex of dimension " << d << endl;
 
    SparseMatrix<Rational> V( ones_vector<Rational>(d+1) | (unit_matrix<Rational>(d) / same_element_vector<Rational>(-1,d)) );
@@ -137,7 +138,7 @@ perl::Object lecture_hall_simplex(int d, perl::OptionSet options)
    if (d <= 0)
       throw std::runtime_error("lecture_hall_simplex : dimension must be postive");
 
-   perl::Object p(perl::ObjectType::construct<Rational>("Polytope"));
+   perl::Object p("Polytope<Rational>");
    p.set_description() << "lecture hall simplex of dimension " << d << endl;
 
    Matrix<Rational> V(d+1,d+1);
@@ -166,9 +167,9 @@ UserFunction4perl("# @category Producing regular polytopes and their generalizat
                   "# | 1 1 0"
                   "# | 1 0 1"
                   "# | 1 1/2-1/2r3 1/2-1/2r3"
-                  "# The polytopes cordinate type is QuadraticExtension<Rational>, thus numbers that can"
+                  "# The polytopes cordinate type is ''QuadraticExtension<Rational>'', thus numbers that can"
                   "# be represented as a + b*sqrt(c) with Rational numbers a, b and c. The last row vectors"
-                  "# entrys thus represent the number 1/2*(1-sqrt(3))."
+                  "# entries thus represent the number $ 1/2 * ( 1 - sqrt(3) ) $."
                   "# @example To store a regular 3-simplex in the variable $s and also calculate its"
                   "# symmetry group, type this:"
                   "# > $s = regular_simplex(3, group=>1);"
