@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2019
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -16,14 +17,41 @@
 
 #define USE_DD_
 #include "polymake/polytope/cdd_interface_impl.h"
-#include <sstream>
 
 namespace polymake { namespace polytope { namespace cdd_interface {
 
-// variable to count the number of solver instances, see cdd_interface_impl.h
-int solver_count = 0;
+#ifdef POLYMAKE_CDD_STANDALONE_GLOBAL_INIT
 
-template <> inline
+namespace {
+
+void global_construct_standalone()
+{
+   dd_set_global_constants();
+}
+
+void global_destroy_standalone()
+{
+   dd_free_global_constants();
+}
+
+}
+
+void (* const CddInstance::Initializer::global_construct)() = &global_construct_standalone;
+void (* const CddInstance::Initializer::global_destroy)() = &global_destroy_standalone;
+
+#endif
+
+CddInstance::Initializer::Initializer()
+{
+   global_construct();
+}
+
+CddInstance::Initializer::~Initializer()
+{
+   global_destroy();
+}
+
+template <>
 Rational cdd_lp_sol<Rational>::optimal_value() const
 {
    return Rational(std::move(ptr->optvalue));
@@ -31,7 +59,8 @@ Rational cdd_lp_sol<Rational>::optimal_value() const
 
 template class cdd_matrix<Rational>;
 template class cdd_polyhedron<Rational>;
-template class solver<Rational>;
+template class ConvexHullSolver<Rational>;
+template class LP_Solver<Rational>;
 
 dd_ErrorType err;
 
